from .predictiq import predictiq_pipeline
# TODO: add all pipelines that should be exposed to the user of your backend in the import statement above.
pipelines = {"predictiq_pipeline": predictiq_pipeline}