from .predictiq import PredictIQBackend
# TODO: add all backend classes that should be exposed to the user of your backend in the import statement above.
backends = {
    "predict_iq": PredictIQBackend
}